from context import signalboxclient
import environment
import time
import uuid
import random

creds = signalboxclient.Credentials(environment.api_key)
config = signalboxclient.Configuration(
    host=environment.host, port=environment.port)
client = signalboxclient.client.SignalBoxClient(creds)

# grab the experimentID from setup_simulation.py
experiment_id = 1
num_presentations = 200

# Agent classes


class Agent:
    def __init__(self, label):
        self.id = str(uuid.uuid1())
        self.label = label


class ProbAgent(Agent):
    def __init__(self, label, acceptance_probs):
        Agent.__init__(self, label)
        self.acceptance_probs = acceptance_probs

    def handle_offers(self, offers):

        if random.uniform(0, 1) <= self.acceptance_probs[offers[0]["name"]]:
            return (offers[0], 'accept')
        else:
            return (offers[0], 'reject')


# initialises type of agent with corresponding prob distribution for acceptance
def get_agent(label):
    if label == 'French':
        return ProbAgent('French', {'offer0': 0.9, 'offer1': 0.875, 'offer2': 0.85, 'offer3': 0.8})
        # return ProbAgent('French', {'offer0': 0, 'offer1': 0, 'offer2': 0, 'offer3': 1})
    elif label == 'German':
        return ProbAgent('German', {'offer0': 0.8, 'offer1': 0.775, 'offer2': 0.75, 'offer3': 0.7})

    raise Error('agent label does not exist')


num_accepted = 0
start = time.time()
print("Starting presentations...")
for i in range(num_presentations):
    agent = get_agent(random.choice(['French', 'German']))
    if (i % 25) == 0:
        print(f"Presenting offer {i}")
    recommendation = client.recommend_offer(
        experiment_id, agent.id, {'country': agent.label})
    (offer, outcome) = agent.handle_offers(recommendation['offers'])
    tracked_outcome = client.track_recommendation_outcome(
        recommendation=recommendation, offerId=offer["id"], outcome=outcome)
    if tracked_outcome['outcome'] == 'accept':
        num_accepted += 1

print(f'{100*num_accepted/num_presentations}% outcomes accepted from {num_presentations} runs')

end = time.time()
print('Time elapsed:', end - start)



from .client_functions import get, post
from .exceptions import SignalBoxException


def create_touchpoint_on_tracked_user(access_token: str, base_url: str, tracked_user_id: str, touchpoint_id: str, values: dict, useInternalId: bool = None):
    json_params = {
        "values": values,
    }
    r = post(f'{base_url}/api/trackedUsers/{tracked_user_id}/touchpoints/{touchpoint_id}?useInternalId={useInternalId or ""}',
             json_params, access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)


def get_touchpoint_on_tracked_user(access_token: str, base_url: str, tracked_user_id: str, touchpoint_id: str, version: int = None, useInternalId: bool = None):
    r = get(f'{base_url}/api/trackedUsers/{tracked_user_id}/touchpoints/{touchpoint_id}?version={version or ""}&useInternalId={useInternalId or ""}', access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)



import requests
from typing import List


def batch(iterable, n=2500):
    l = len(iterable)
    for ndx in range(0, l, n):
        yield iterable[ndx:min(ndx + n, l)]


def post_api_header(access_token):
    return {"Content-Type": "application/json", "Authorization": f'Bearer {access_token}'}


def get_api_header(access_token):
    return {"Authorization": f'Bearer {access_token}'}


def post(api_endpoint, json_params, access_token):
    verify = "localhost" not in api_endpoint
    return requests.post(url=api_endpoint, json=json_params, headers=post_api_header((access_token)), verify=verify)


def put(api_endpoint, json_params, access_token):
    verify = "localhost" not in api_endpoint
    return requests.put(url=api_endpoint, json=json_params, headers=post_api_header((access_token)), verify=verify)


def get(api_endpoint, access_token):
    verify = "localhost" not in api_endpoint
    return requests.get(url=api_endpoint, headers=get_api_header((access_token)), verify=verify)


def construct_event(commonUserId, event_id, event_type, kind, properties, timestamp=None, source_system_id=None):
    return {
        'commonUserId': commonUserId,
        'eventId': event_id,
        'eventType': event_type,
        'kind': kind,
        'properties': properties,
        'timestamp': timestamp,
        'sourceSystemId': source_system_id,
    }


def construct_user(commonUserId: str, name: str = None, properties: dict = None, integratedSystemId: int = None, userId: str = None):
    model = {
        'commonUserId': commonUserId,
        'name': name,
        'properties': properties,
    }
    if(integratedSystemId is not None):
        assert userId is not None
        model['integratedSystemReference'] = {
            'integratedSystemId': integratedSystemId,
            'userId': str(userId) # must be a string
        }

    return model




def pageQuery(page: int = None):
    return f'p.page={1 if page is None else page}'


class Pagination:
    def __init__(self, pageCount, totalItemCount, pageNumber, hasPreviousPage, hasNextPage, isFirstPage, isLastPage):
        self.pageCount = pageCount
        self.totalItemCount = totalItemCount
        self.pageNumber = pageNumber
        self.hasPreviousPage = hasPreviousPage
        self.hasNextPage = hasNextPage
        self.isFirstPage = isFirstPage
        self.isLastPage = isLastPage

class PaginatedResponse:
    def __init__(self, **kwargs):
        self.items = kwargs['items']
        self.pagination = Pagination(**kwargs['pagination'])


from context import signalboxclient
import environment

creds = signalboxclient.Credentials(environment.api_key)
config = signalboxclient.Configuration(
    host=environment.host, port=environment.port)
client = signalboxclient.client.SignalBoxClient(creds)

# create some offers
offer_ids = []
for i in range(4):
    offer = client.create_offer(name=f'offer{i}', currency='AUD', price=(i+1)*5, cost=(i+1)*2)
    offer_ids.append(offer['id'])

# create an experiment
experiment = client.create_experiment(offer_ids, name='sim_experiment')

# export/print the experimentID
print(f'experiment id: {experiment["id"]}')



from dotenv import dotenv_values

config = dotenv_values(".local.env")

api_key = config.get('api_key')
host = config.get('host')
port = config.get('port')

print('host is ', host)
print('port is ', port)


from .client_functions import batch, get, post, put
from .pagination import pageQuery, PaginatedResponse
from .exceptions import SignalBoxException


def create_user(access_token, base_url: str, name: str, common_user_id: str):
    json_params = {
        "name": name,
        "commonUserId": common_user_id
    }
    r = post(f'{base_url}/api/trackedUsers',
             json_params, access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)


def create_or_update_users(access_token: str, base_url: str, users):
    for u in users:
        if 'commonUserId' not in u:
            raise Exception("Tracked Users require a commonUserId")
    results = []
    for u in batch(users):
        json_params = {
            "users": u
        }
        r = put(f'{base_url}/api/trackedUsers', json_params, access_token)
        if r.ok:
            results.append(r.json())
        else:
            raise SignalBoxException(r.text)
    return results



from .client_functions import get, post
from .pagination import pageQuery, PaginatedResponse
from .exceptions import SignalBoxException


def create_experiment(access_token: str, base_url: str, offer_ids: list, name: str, concurrent_offers: int = 1, segment_id: str = None):
    json_params = {
        "offerIds": offer_ids,
        "segmentId": segment_id,
        "name": name,
        "concurrentOffers": concurrent_offers
    }
    r = post(f'{base_url}/api/experiments',
             json_params, access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)


def get_experiment(access_token: str, base_url: str, experiment_id: str):
    r = get(f'{base_url}/api/experiments/{experiment_id}', access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)


def query_experiments(access_token: str, base_url: str, page: int = None):
    r = get(f'{base_url}/api/experiments?{pageQuery(page)}', access_token)
    if r.ok:
        return PaginatedResponse(**r.json())
    else:
        raise SignalBoxException(r.text)


def get_offers_in_experiment(access_token: str, base_url: str, experiment_id: str):
    r = get(
        f'{base_url}/api/experiments/{experiment_id}/offers', access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)



"""A library used as a Four2/SignalBox client."""

__version__ = '0.1.11'
from .client import SignalBoxClient, Credentials, Configuration
from .client_functions import construct_user, construct_event



from .client_functions import get, post
from .pagination import pageQuery, PaginatedResponse
from .exceptions import SignalBoxException


def create_product(access_token: str, base_url: str, name: str, product_id: str, description: str):
    json_params = {
        "name": name,
        "productId": product_id,
        "description": description,
    }
    r = post(f'{base_url}/api/products',
             json_params, access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)

def query_products(access_token: str, base_url: str, page: int = None):
    r = get(f'{base_url}/api/products?{pageQuery(page)}', access_token)
    if r.ok:
        return PaginatedResponse(**r.json())
    else:
        raise SignalBoxException(r.text)


from context import signalboxclient
# import client
import datetime
import uuid
import numpy as np
import environment

creds = signalboxclient.client.Credentials(environment.api_key)
test_client = signalboxclient.client.SignalBoxClient(creds)
my_token = test_client.access_token

# create user
user_id = str(uuid.uuid1())
u = test_client.create_user('testuser', user_id)
print('created user with id=', user_id)
# update the user
u['properties'] = {
    'client_test.py': datetime.datetime.now().strftime('%Y-%m-%dT%H:%M:%S.%f%z')
}
test_client.create_or_update_users([u])
offer = test_client.create_offer('my_offer', 'AUD', 100, 50)
offer_id = offer['id']

# create a touchpoint
touchpoint_id = "client_test.py"
tp = test_client.create_touchpoint_on_tracked_user(
    user_id, touchpoint_id, {"key": "value"})

assert tp['version'] > 0
touchpoint_id = "client_test.py"
tp = test_client.get_touchpoint_on_tracked_user(
    user_id, touchpoint_id)

assert tp["values"]["key"] == "value"
# get the touchpoint


# create a product
# TODO: next sprint
# product = test_client.create_product("Test Product Name", str(
#     uuid.uuid1()), "A test description.")
# assert product['id'] > 0

# # query the products
# response = test_client.query_products(page=1)

# assert len(response.items) > 1

experiment = test_client.create_experiment(
    [offer_id], name='Experiment Name', segment_id=None, concurrent_offers=1)
print('created experiment')
experiment_id = experiment['id']

# list all experiments
experiments_first_page = test_client.query_experiments(1)
print('len experiments_first_page =', len(experiments_first_page.items))
assert experiments_first_page.pagination.totalItemCount > 1

get_experiment_response = test_client.get_experiment(experiment_id)
print('\nPrinting get_experiment test response: ', get_experiment_response)

offers_in_experiment = test_client.get_offers_in_experiment(experiment_id)
assert offers_in_experiment is not None
assert len(offers_in_experiment) > 0

print('\nPrinting create_offer test response: ', offer)
print('\nPrinting create_experiment test response: ', experiment)

presentation = test_client.recommend_offer(experiment_id, user_id)
print(presentation)

# choose accept or rekect
tracked = test_client.track_recommendation_outcome(
    presentation, presentation['offers'][0]['id'], 'accept')
print(tracked)

event_types = ["CREATED", "CANCELED"]
event_kinds = ["BILLING", "TICKET"]
events = []
for i in range(100):
    events.append(test_client.construct_event(
        str(uuid.uuid1()), str(uuid.uuid1()), np.random.choice(event_types), np.random.choice(event_kinds), {}))
    # commonUserId, event_id, event_type, kind, properties, timestamp, source_system_id

print(f'logging {len(events)} events')
result = test_client.log_events(events)
print(result)

print('\ntests successful')



class CredentialsException(Exception):
    def __init__(self, message):
        # Call the base class constructor with the parameters it needs
        super().__init__(message)


class SignalBoxException(Exception):
    def __init__(self, message):
        # Call the base class constructor with the parameters it needs
        super().__init__(message)
        print(message)



import requests
import warnings
from .exceptions import CredentialsException, SignalBoxException
from .client_functions import batch, construct_user, construct_event, post, put
from .experiments import create_experiment, get_experiment, get_offers_in_experiment, query_experiments
from .products import create_product, query_products
from .trackedusers import create_user, create_or_update_users
from .touchpoints import create_touchpoint_on_tracked_user, get_touchpoint_on_tracked_user


class Configuration:
    def __init__(self, host='localhost', port=5001, protocol='https'):
        self.host = host
        self.port = port
        self.protocol = protocol
        if "localhost" in host:
            warnings.filterwarnings(
                "ignore", message="Unverified HTTPS request is being made to host")

    def base_url(self):
        return f'{self.protocol}://{self.host}:{self.port}'


class Credentials:
    def __init__(self, api_key=None, access_token=None):
        self.api_key = api_key
        self.access_token = access_token
        if api_key is None and access_token is None:
            raise CredentialsException(
                "one of api_key or access_token must not be null")

    def fetch_token(self, configuration: Configuration):
        if self.access_token is None:
            api_endpoint = f'{configuration.base_url()}/api/apikeys/exchange'
            api_headers = {"Content-Type": "application/json"}
            json_params = {
                "apiKey": self.api_key,
            }

            r = requests.post(url=api_endpoint, json=json_params,
                              headers=api_headers, verify=False)

            if r.ok:
                self.access_token = r.json()['access_token']
                if self.access_token == None:
                    raise CredentialsException("Access Token is None")

            else:
                raise CredentialsException(
                    f'Server returned: {r.status_code}, with body: {r.text}')
        return self.access_token


class SignalBoxClient:
    def __init__(self, credentials: Credentials, configuration: Configuration = None):
        if configuration is None:
            configuration = Configuration()
        self.access_token = credentials.fetch_token(
            configuration=configuration)
        self.base_url = configuration.base_url()

    def create_offer(self, name: str, currency: str, price: float, cost: float):
        json_params = {
            "name": name,
            "currency": currency,
            "price": price,
            "cost": cost
        }

        r = post(f'{self.base_url}/api/offers', json_params, self.access_token)
        if r.ok:
            return r.json()
        else:
            raise SignalBoxException(r.text)

    def create_user(self, name: str, common_user_id: str):
        return create_user(self.access_token, self.base_url, name=name, common_user_id=common_user_id)

    def create_touchpoint_on_tracked_user(self, tracked_user_id: str, touchpoint_id: str, values: dict):
        return create_touchpoint_on_tracked_user(
            self.access_token, self.base_url, tracked_user_id=tracked_user_id, touchpoint_id=touchpoint_id, values=values)

    def get_touchpoint_on_tracked_user(self, tracked_user_id: str, touchpoint_id: str, version: int = None):
        return get_touchpoint_on_tracked_user(
            self.access_token, self.base_url, tracked_user_id=tracked_user_id, touchpoint_id=touchpoint_id, version=version)

    def create_or_update_users(self, users):
        return create_or_update_users(self.access_token, self.base_url,  users)

    def create_product(self, name: str, product_id: str, description: str):
        return create_product(self.access_token, self.base_url, name, product_id, description)

    def query_products(self, page: int):
        return query_products(self.access_token, self.base_url, page)

    def create_segment(self, name: str):
        json_params = {
            "name": name
        }
        r = post(f'{self.base_url}/api/segments',
                 json_params, self.access_token)

        if r.ok:
            return r.json()
        else:
            raise SignalBoxException(r.text)

    def create_experiment(self, offer_ids: list, name: str, concurrent_offers: int = 1, segment_id: str = None):
        return create_experiment(self.access_token, self.base_url,
                                 offer_ids, name, concurrent_offers, segment_id)

    def get_experiment(self, experiment_id: str):
        return get_experiment(self.access_token, self.base_url, experiment_id)

    def query_experiments(self, page: int = None):
        return query_experiments(self.access_token, self.base_url, page=page)

    def get_offers_in_experiment(self, experiment_id: str):
        return get_offers_in_experiment(self.access_token, self.base_url, experiment_id)

    def recommend_offer(self, experimentId: str, commonUserId: str, features: dict = None):
        json_params = {
            "commonUserId": commonUserId,
            "features": features
        }
        r = post(f'{self.base_url}/api/experiments/{experimentId}/recommendation',
                 json_params, self.access_token)

        if r.ok:
            return r.json()
        else:
            raise SignalBoxException(r.text)

    def track_recommendation_outcome(self, recommendation, offerId, outcome):
        json_params = {
            "commonUserId":  recommendation['commonUserId'],
            'experimentId': recommendation['experimentId'],
            'iterationId': recommendation['iterationId'],
            'recommendationId': recommendation['recommendationId'],
            'offerId': offerId,
            'outcome': outcome
        }
        r = post(
            f'{self.base_url}/api/experiments/{recommendation["experimentId"]}/outcome', json_params, self.access_token)

        if r.ok:
            return r.json()
        else:
            raise SignalBoxException(r.text)

    def construct_event(self, commonUserId, event_id, event_type, kind, properties, timestamp=None, source_system_id=None):
        return construct_event(commonUserId, event_id, event_type, kind, properties, timestamp, source_system_id)

    def construct_user(self, commonUserId: str, name: str = None, properties: str = None, integratedSystemId: int = None, userId: str = None):
        return construct_user(commonUserId, name=name, properties=properties, integratedSystemId=integratedSystemId, userId=userId)

    def log_events(self, events):
        results = []
        for e in batch(events):
            r = post(f'{self.base_url}/api/events', e, self.access_token)
            if(r.ok):
                results.append(r.json())
            else:
                raise SignalBoxException(r.text)
        return results
